/////////////////////////<Source Code Embedded Notices>/////////////////////////
//
// INTEL CONFIDENTIAL
// Copyright (C) Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to
// the source code ("Material") are owned by Intel Corporation or its suppliers
// or licensors. Title to the Material remains with Intel Corporation or its
// suppliers and licensors. The Material contains trade secrets and proprietary
// and confidential information of Intel or its suppliers and licensors. The
// Material is protected by worldwide copyright and trade secret laws and
// treaty provisions. No part of the Material may be used, copied, reproduced,
// modified, published, uploaded, posted, transmitted, distributed, or disclosed
// in any way without Intel's prior express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual
// property right is granted to or conferred upon you by disclosure or delivery
// of the Materials, either expressly, by implication, inducement, estoppel or
// otherwise. Any license under such intellectual property rights must be
// express and approved by Intel in writing.
//
/////////////////////////<Source Code Embedded Notices>/////////////////////////
///  @file
///
///  @brief Implements functionality for accessing information about the current execution environment
//////////////////////////////////////////////////////////////////////////////

#pragma once

#include <Foundation/Error/Error.h>

#include <string>

namespace CommonUtils
{
    /*!
    @brief Gets the path to the user's home directory

    Examples:
        - C:\\Users\\bob
        - /home/bob
        - /Users/bob
    */
    OpenIPC_Error GetUserHomeDirectory(OUT std::string& directory);

    /*!
    @brief Gets the name of the current process and its PID
    */
    OpenIPC_Error GetCurrentProcessNameAndId(OUT std::string& processName, OUT uint32_t& processID);

    /*!
    @brief Gets the hostname of the current machine
    */
    OpenIPC_Error GetHostname(OUT std::string& hostname);

    /*!
    @brief Replaces occurances of the user directory with %USERPROFILE% (or ~ on linux/mac).
    */
    OpenIPC_Error CleanUserDirectoryFromString(OUT std::string& out, IN std::string const& in);

    /*!
    @brief Replaces occurances of the host name with %COMPUTERNAME% (or $hostname on linux/mac).
    */
    OpenIPC_Error CleanHostnameFromString(OUT std::string& out, IN std::string const& in);

    /*!
     @brief Gets the path to the system-wide application data directory for OpenIPC

    Examples:
        C:\\ProgramData\\Intel\\OpenIPC
        /etc/intel/openipc
     */
    OpenIPC_Error GetCommonApplicationDataDirectory(OUT std::string& directory);

}
